import csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc
import csle_collector.ossec_ids_manager.ossec_ids_manager_pb2
import csle_collector.constants.constants as constants


def get_ossec_ids_monitor_status(
        stub: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc.OSSECIdsManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
    """
    Queries the IDS manager for the status of the IDS monitor thread

    :param stub: the stub to send the remote gRPC to the server
    :return: an IdsMonitorDTO describing the status of the IDS monitor thread
    """
    get_ids_monitor_status_msg = \
        csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.GetOSSECIdsMonitorStatusMsg()
    ids_monitor_dto: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO = \
        stub.getOSSECIdsMonitorStatus(get_ids_monitor_status_msg, timeout=timeout)
    return ids_monitor_dto


def start_ossec_ids_monitor(stub: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc.OSSECIdsManagerStub,
                            kafka_ip: str, kafka_port: int, log_file_path: str, time_step_len_seconds: int,
                            timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
    """
    Sends a request to the IDS manager to start the IDS monitor thread

    :param kafka_ip: the ip of the Kafka server
    :param kafka_port: the port of the Kafka server
    :param log_file_path: the path top the IDS log
    :param time_step_len_seconds: the length of one time-step
    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an IdsMonitorDTO describing the status of the IDS monitor thread
    """
    start_ids_monitor_msg = csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.StartOSSECIdsMonitorMsg(
        kafka_ip=kafka_ip, kafka_port=kafka_port, log_file_path=log_file_path,
        time_step_len_seconds=time_step_len_seconds
    )
    ids_monitor_dto: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO = \
        stub.startOSSECIdsMonitor(start_ids_monitor_msg, timeout=timeout)
    return ids_monitor_dto


def stop_ossec_ids_monitor(
        stub: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc.OSSECIdsManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
    """
    Sends a request to the IDS manager to stop the IDS monitor thread

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an IdsMonitorDTO describing the status of the IDS monitor thread
    """
    stop_ids_monitor_msg = \
        csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.StopOSSECIdsMonitorMsg()
    ids_monitor_dto: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO = \
        stub.stopOSSECIdsMonitor(stop_ids_monitor_msg, timeout=timeout)
    return ids_monitor_dto


def stop_ossec_ids(
        stub: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc.OSSECIdsManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
    """
    Sends a request to the OSSEC IDS manager to stop the IDS

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an OSSECIdsMonitorDTO describing the status of the IDS and its monitor thread
    """
    stop_ossec_ids_msg = \
        csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.StopOSSECIdsMsg()
    ids_monitor_dto: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO = \
        stub.stopOSSECIds(stop_ossec_ids_msg, timeout=timeout)
    return ids_monitor_dto


def start_ossec_ids(
        stub: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc.OSSECIdsManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
    """
    Sends a request to the OSSEC IDS manager to start the IDS

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an OSSECIdsMonitorDTO describing the status of the IDS and its monitor thread
    """
    start_ossec_ids_msg = \
        csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.StartOSSECIdsMsg()
    ids_monitor_dto: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO = \
        stub.startOSSECIds(start_ossec_ids_msg, timeout=timeout)
    return ids_monitor_dto


def get_ossec_ids_alerts(
        stub: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc.OSSECIdsManagerStub, timestamp: float,
        log_file_path: str, timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO:
    """
    Queries the IDS manager for the data of the IDS log from a given time-step

    :param stub: the stub to send the remote gRPC to the server
    :param timestamp: the timtestamp to parse the log from
    :param log_file_path: path to the IDS log file to read
    :param timeout: the GRPC timeout (seconds)
    :return: an IdsLogDTO with data of the IDS log
    """
    get_ids_log_alerts_msg = \
        csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.GetOSSECIdsAlertsMsg(
            timestamp=timestamp, log_file_path=log_file_path)
    ids_log_dto: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO = \
        stub.getOSSECIdsAlerts(get_ids_log_alerts_msg, timeout=timeout)
    return ids_log_dto



from typing import Any
import pytest
import pytest_mock
import csle_collector.host_manager.query_host_manager
from csle_collector.host_manager.host_manager_pb2 import HostStatusDTO, HostMetricsDTO
from csle_collector.host_manager.host_manager import HostManagerServicer
from csle_collector.host_manager.dao.host_metrics import HostMetrics


class TestHostManagerSuite:
    """
    Test suite for the Host manager
    """

    @pytest.fixture(scope='module')
    def grpc_add_to_server(self) -> Any:
        """
        Necessary fixture for pytest-grpc

        :return: the add_servicer_to_server function
        """
        from csle_collector.host_manager.host_manager_pb2_grpc import add_HostManagerServicer_to_server
        return add_HostManagerServicer_to_server

    @pytest.fixture(scope='module')
    def grpc_servicer(self) -> HostManagerServicer:
        """
        Necessary fixture for pytest-grpc

        :return: the host manager servicer
        """
        return HostManagerServicer()

    @pytest.fixture(scope='module')
    def grpc_stub_cls(self, grpc_channel):
        """
        Necessary fixture for pytest-grpc

        :param grpc_channel: the grpc channel for testing
        :return: the stub to the service
        """
        from csle_collector.host_manager.host_manager_pb2_grpc import HostManagerStub
        return HostManagerStub

    def test_startHostMonitor(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startHostMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.threads.host_monitor_thread.HostMonitorThread.run',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.threads.host_monitor_thread.HostMonitorThread.__init__',
                     return_value=None)
        mocker.patch('csle_collector.host_manager.threads.host_monitor_thread.HostMonitorThread.start',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        kafka_ip = "test_kafka_ip"
        kafka_port = 9292
        time_step_len_seconds = 30
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.start_host_monitor(
            stub=grpc_stub, kafka_ip=kafka_ip, kafka_port=kafka_port, time_step_len_seconds=time_step_len_seconds)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=False)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=False)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=False)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=False)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=False)
        response = csle_collector.host_manager.query_host_manager.start_host_monitor(
            stub=grpc_stub, kafka_ip=kafka_ip, kafka_port=kafka_port, time_step_len_seconds=time_step_len_seconds)
        assert response.monitor_running
        assert not response.filebeat_running
        assert not response.packetbeat_running
        assert not response.metricbeat_running
        assert not response.heartbeat_running

    def test_stopHostMonitor(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopHostMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.threads.host_monitor_thread.HostMonitorThread.run',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.threads.host_monitor_thread.HostMonitorThread.__init__',
                     return_value=None)
        mocker.patch('csle_collector.host_manager.threads.host_monitor_thread.HostMonitorThread.start',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.stop_host_monitor(stub=grpc_stub)
        assert not response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=False)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=False)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=False)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=False)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=False)
        response = csle_collector.host_manager.query_host_manager.stop_host_monitor(stub=grpc_stub)
        assert not response.monitor_running
        assert not response.filebeat_running
        assert not response.packetbeat_running
        assert not response.metricbeat_running
        assert not response.heartbeat_running

    def test_startFilebeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startFilebeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._start_filebeat',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.start_filebeat(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_stopFilebeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopFilebeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._stop_filebeat',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.stop_filebeat(stub=grpc_stub)
        assert response.monitor_running
        assert not response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_configFilebeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the configFilebeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._set_filebeat_config',
                     return_value=True)
        log_files_paths = ["test_path"]
        kibana_ip = "test_kibana_ip"
        kibana_port = 999
        elastic_ip = "test_elastic_ip"
        elastic_port = 333
        num_elastic_shards = 415
        kafka_topics = ["topic1"]
        kafka_ip = "test_kafka_ip"
        kafka_port = 5125
        filebeat_modules = ["mod1", "mod2"]
        reload_enabled = False
        kafka = True
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.config_filebeat(
            stub=grpc_stub, log_files_paths=log_files_paths, kibana_ip=kibana_ip, kibana_port=kibana_port,
            elastic_ip=elastic_ip, elastic_port=elastic_port, num_elastic_shards=num_elastic_shards,
            kafka_topics=kafka_topics, kafka_port=kafka_port, filebeat_modules=filebeat_modules,
            reload_enabled=reload_enabled, kafka=kafka, kafka_ip=kafka_ip)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_startMetricbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startMetricbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._start_metricbeat',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.start_metricbeat(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_stopMetricbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopMetricbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._stop_metricbeat',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.stop_metricbeat(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert not response.metricbeat_running
        assert response.heartbeat_running

    def test_configMetricbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the configMetricbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._set_metricbeat_config',
                     return_value=True)
        kibana_ip = "test_kibana_ip"
        kibana_port = 999
        elastic_ip = "test_elastic_ip"
        elastic_port = 333
        num_elastic_shards = 415
        kafka_ip = "test_kafka_ip"
        kafka_port = 5125
        metricbeat_modules = ["mod1", "mod2"]
        reload_enabled = False
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.config_metricbeat(
            stub=grpc_stub, kibana_ip=kibana_ip, kibana_port=kibana_port,
            elastic_ip=elastic_ip, elastic_port=elastic_port, num_elastic_shards=num_elastic_shards,
            kafka_port=kafka_port, metricbeat_modules=metricbeat_modules,
            reload_enabled=reload_enabled, kafka_ip=kafka_ip)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_startPacketbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startPacketbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._start_packetbeat',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.start_packetbeat(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_stopPacketbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopPacketbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._stop_packetbeat',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.stop_packetbeat(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert not response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_configPacketbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the configPacketbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._set_packetbeat_config',
                     return_value=True)
        kibana_ip = "test_kibana_ip"
        kibana_port = 999
        elastic_ip = "test_elastic_ip"
        elastic_port = 333
        num_elastic_shards = 415
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.config_packetbeat(
            stub=grpc_stub, kibana_ip=kibana_ip, kibana_port=kibana_port,
            elastic_ip=elastic_ip, elastic_port=elastic_port, num_elastic_shards=num_elastic_shards)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_startHeartbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startHeartbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._start_heartbeat',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.start_heartbeat(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_stopHeartbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopPacketbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._stop_heartbeat',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.stop_heartbeat(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert not response.heartbeat_running

    def test_configHeartbeat(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the configHeartbeat grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._set_heartbeat_config',
                     return_value=True)
        kibana_ip = "test_kibana_ip"
        kibana_port = 999
        elastic_ip = "test_elastic_ip"
        elastic_port = 333
        num_elastic_shards = 415
        hosts_to_monitor = ["host1", "host2"]
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.config_heartbeat(
            stub=grpc_stub, kibana_ip=kibana_ip, kibana_port=kibana_port,
            elastic_ip=elastic_ip, elastic_port=elastic_port, num_elastic_shards=num_elastic_shards,
            hosts_to_monitor=hosts_to_monitor)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_startSpark(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startSpark grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._start_spark',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.start_spark(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_stopSpark(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopSpark grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._stop_spark',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.stop_spark(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_getHostStatus(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the getHostStatus grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_filebeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_packetbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_metricbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._get_heartbeat_status',
                     return_value=True)
        mocker.patch('csle_collector.host_manager.host_manager.HostManagerServicer._is_monitor_running',
                     return_value=True)
        response: HostStatusDTO = csle_collector.host_manager.query_host_manager.get_host_status(stub=grpc_stub)
        assert response.monitor_running
        assert response.filebeat_running
        assert response.packetbeat_running
        assert response.metricbeat_running
        assert response.heartbeat_running

    def test_getHostMetrics(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the getHostMetrics grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        num_logged_in_users = 23
        num_failed_login_attempts = 99
        num_open_connections = 3
        num_login_events = 71
        num_processes = 31
        num_users = 6
        ip = "test_ip"
        ts = 0.0
        host_metrics = HostMetrics(
            num_logged_in_users=num_logged_in_users, num_failed_login_attempts=num_failed_login_attempts,
            num_open_connections=num_open_connections, num_login_events=num_login_events,
            num_processes=num_processes, num_users=num_users, ip=ip, ts=ts)
        mocker.patch('csle_collector.host_manager.host_manager_util.HostManagerUtil.read_host_metrics',
                     return_value=host_metrics)
        response: HostMetricsDTO = csle_collector.host_manager.query_host_manager.get_host_metrics(
            stub=grpc_stub, login_last_ts=0.0, failed_auth_last_ts=0.0)
        assert response.num_logged_in_users == num_logged_in_users
        assert response.num_failed_login_attempts == num_failed_login_attempts
        assert response.num_open_connections == num_open_connections
        assert response.num_login_events == num_login_events
        assert response.num_processes == num_processes
        assert response.num_users == num_users



from typing import Union
import time
import logging
import grpc
from concurrent import futures
from csle_collector.client_manager.dao.client import Client
from csle_collector.client_manager.threads.arrival_thread import ArrivalThread
from csle_collector.client_manager.threads.producer_thread import ProducerThread
from csle_collector.client_manager.dao.workflows_config import WorkflowsConfig
import csle_collector.client_manager.client_manager_pb2_grpc
import csle_collector.client_manager.client_manager_pb2
import csle_collector.constants.constants as constants


class ClientManagerServicer(csle_collector.client_manager.client_manager_pb2_grpc.ClientManagerServicer):
    """
    gRPC server for managing the running clients. Allows to start/stop clients remotely and also to query the
    state of the clients.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        self.arrival_thread: Union[None, ArrivalThread] = None
        self.producer_thread: Union[None, ProducerThread] = None
        logging.basicConfig(filename=f"{constants.LOG_FILES.CLIENT_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.CLIENT_MANAGER_LOG_FILE}", level=logging.INFO)

    def get_arrival_thread(self) -> Union[None, ArrivalThread]:
        """
        Gets the arrival thread

        :return: The arrival thread if it is initialized, otherwise None
        """
        return self.arrival_thread

    def get_producer_thread(self) -> Union[None, ProducerThread]:
        """
        Gets the producer thread

        :return: The producer thread if it is initialized, otherwise None
        """
        return self.producer_thread

    def getClients(self, request: csle_collector.client_manager.client_manager_pb2.GetClientsMsg,
                   context: grpc.ServicerContext) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Gets the state of the clients

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        num_clients = 0
        clients_time_step_len_seconds = 0.0
        producer_time_step_len_seconds = 0.0
        arrival_thread = self.get_arrival_thread()
        producer_thread = self.get_producer_thread()

        client_process_active = False
        if arrival_thread is not None:
            num_clients = len(arrival_thread.client_threads)
            client_process_active = True
            clients_time_step_len_seconds = arrival_thread.time_step_len_seconds

        producer_active = False
        if producer_thread is not None:
            producer_active = True
            producer_time_step_len_seconds = producer_thread.time_step_len_seconds

        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=num_clients, client_process_active=client_process_active, producer_active=producer_active,
            clients_time_step_len_seconds=int(clients_time_step_len_seconds),
            producer_time_step_len_seconds=int(producer_time_step_len_seconds))
        return clients_dto

    def stopClients(self, request: csle_collector.client_manager.client_manager_pb2.StopClientsMsg,
                    context: grpc.ServicerContext):
        """
        Stops the Poisson-process that generates new clients

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        logging.info("Stopping clients")

        clients_time_step_len_seconds = 0.0
        producer_time_step_len_seconds = 0.0
        producer_thread = self.get_producer_thread()
        arrival_thread = self.get_producer_thread()

        if arrival_thread is not None:
            clients_time_step_len_seconds = arrival_thread.time_step_len_seconds
            arrival_thread.stopped = True
            time.sleep(1)
        self.arrival_thread = None

        producer_active = False
        if producer_thread is not None:
            producer_active = True
            producer_time_step_len_seconds = producer_thread.time_step_len_seconds

        return csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=0, client_process_active=False, producer_active=producer_active,
            clients_time_step_len_seconds=int(clients_time_step_len_seconds),
            producer_time_step_len_seconds=int(producer_time_step_len_seconds))

    def startClients(self, request: csle_collector.client_manager.client_manager_pb2.StartClientsMsg,
                     context: grpc.ServicerContext) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Starts/Restarts the Poisson process(es) that generate(s) clients

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        clients = list(map(lambda x: Client.from_grpc_object(x), request.clients))
        workflows_config = WorkflowsConfig.from_grpc_object(request.workflows_config)
        logging.info(f"Starting clients, num clients:{len(clients)}, "
                     f"num workflows: {len(workflows_config.workflow_markov_chains)}, "
                     f"num services: {len(workflows_config.workflow_services)}, "
                     f"client types: {list(map(lambda x: str(x), clients))},"
                     f"workflow markov chains: {list(map(lambda x: str(x), workflows_config.workflow_markov_chains))},"
                     f"workflow services: {list(map(lambda x: str(x), workflows_config.workflow_services))},"
                     f"\n commands: {workflows_config.commands()}")
        producer_time_step_len_seconds = 0
        arrival_thread = self.get_arrival_thread()
        producer_thread = self.get_producer_thread()
        if arrival_thread is not None:
            arrival_thread.stopped = True
            time.sleep(1)
        if request.time_step_len_seconds <= 0:
            request.time_step_len_seconds = 1
        arrival_thread = ArrivalThread(time_step_len_seconds=request.time_step_len_seconds, clients=clients,
                                       workflows_config=workflows_config)
        arrival_thread.start()
        self.arrival_thread = arrival_thread
        clients_time_step_len_seconds = request.time_step_len_seconds
        producer_active = False
        if producer_thread is not None:
            producer_active = True
            producer_time_step_len_seconds = producer_thread.time_step_len_seconds
        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=0, client_process_active=True, producer_active=producer_active,
            clients_time_step_len_seconds=int(clients_time_step_len_seconds),
            producer_time_step_len_seconds=producer_time_step_len_seconds)
        return clients_dto

    def startProducer(self, request: csle_collector.client_manager.client_manager_pb2.StartProducerMsg,
                      context: grpc.ServicerContext) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Starts/Restarts the producer thread that pushes data to Kafka

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        clients_time_step_len_seconds = 0.0
        producer_thread = self.get_producer_thread()
        arrival_thread = self.get_arrival_thread()
        logging.info(f"Starting producer, time-step len:{request.time_step_len_seconds}s, "
                     f"arrival_thread: {arrival_thread}")

        time.sleep(5)
        if producer_thread is not None:
            producer_thread.stopped = True
            time.sleep(1)
        if request.time_step_len_seconds <= 0:
            request.time_step_len_seconds = 1
        if arrival_thread is None:
            raise ValueError("Cannot start producer if the arrival thread is not started")
        producer_thread = ProducerThread(arrival_thread=arrival_thread,
                                         time_step_len_seconds=request.time_step_len_seconds,
                                         ip=request.ip, port=request.port)
        producer_thread.start()
        self.producer_thread = producer_thread
        client_process_active = False
        num_clients = 0
        if arrival_thread is not None:
            num_clients = len(arrival_thread.client_threads)
            client_process_active = True
            clients_time_step_len_seconds = arrival_thread.time_step_len_seconds
        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=num_clients, client_process_active=client_process_active, producer_active=True,
            clients_time_step_len_seconds=int(clients_time_step_len_seconds),
            producer_time_step_len_seconds=request.time_step_len_seconds)
        return clients_dto

    def stopProducer(self, request: csle_collector.client_manager.client_manager_pb2.StopProducerMsg,
                     context: grpc.ServicerContext) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Stops the producer thread that pushes data to Kafka

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        logging.info("Stopping producer")
        clients_time_step_len_seconds = 0.0
        producer_thread = self.get_producer_thread()
        arrival_thread = self.get_arrival_thread()

        if producer_thread is not None:
            producer_thread.stopped = True
            time.sleep(1)
        self.producer_thread = None
        client_process_active = False
        num_clients = 0
        if arrival_thread is not None:
            num_clients = len(arrival_thread.client_threads)
            client_process_active = True
            clients_time_step_len_seconds = arrival_thread.time_step_len_seconds
        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=num_clients, client_process_active=client_process_active, producer_active=False,
            clients_time_step_len_seconds=int(clients_time_step_len_seconds), producer_time_step_len_seconds=0)
        return clients_dto


def serve(port: int = 50044, log_dir: str = "/", log_file_name: str = "client_manager.log",
          max_workers: int = 10) -> None:
    """
    Starts the gRPC server for managing clients

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.CLIENT_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.CLIENT_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.client_manager.client_manager_pb2_grpc.add_ClientManagerServicer_to_server(
        ClientManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"ClientManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



from typing import Dict, Any
import csle_collector.ryu_manager.ryu_manager_pb2


class RyuManagerUtil:
    """
    Class with utility functions related to the Ryu Manager
    """

    @staticmethod
    def ryu_dto_to_dict(ryu_dto: csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO) -> Dict[str, Any]:
        """
        Converts a RyuDTO to a dict

        :param ryu_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d: Dict[str, Any] = {}
        d["ryu_running"] = ryu_dto.ryu_running
        d["monitor_running"] = ryu_dto.monitor_running
        d["port"] = ryu_dto.port
        d["web_port"] = ryu_dto.web_port
        d["controller"] = ryu_dto.controller
        d["kafka_ip"] = ryu_dto.kafka_ip
        d["kafka_port"] = ryu_dto.kafka_port
        d["time_step_len"] = ryu_dto.time_step_len
        return d

    @staticmethod
    def ryu_dto_from_dict(d: Dict[str, Any]) -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
        """
        Converts a dict representation of a RyuDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        ryu_dto = csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO()
        ryu_dto.ryu_running = d["ryu_running"]
        ryu_dto.monitor_running = d["monitor_running"]
        ryu_dto.port = d["port"]
        ryu_dto.web_port = d["web_port"]
        ryu_dto.controller = d["controller"]
        ryu_dto.kafka_ip = d["kafka_ip"]
        ryu_dto.kafka_port = d["kafka_port"]
        ryu_dto.time_step_len = d["time_step_len"]
        return ryu_dto

    @staticmethod
    def ryu_dto_empty() -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
        """
        :return: an empty RyuDTO
        """
        ryu_dto = csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO()
        ryu_dto.ryu_running = False
        ryu_dto.monitor_running = False
        ryu_dto.port = 6633
        ryu_dto.web_port = 8080
        ryu_dto.kafka_ip = ""
        ryu_dto.kafka_port = 9092
        ryu_dto.controller = ""
        ryu_dto.time_step_len = 30
        return ryu_dto



from typing import Dict, Any, List, Union
import datetime
import subprocess
import yaml
import logging
import csle_collector.constants.constants as constants
from csle_collector.host_manager.dao.failed_login_attempt import FailedLoginAttempt
from csle_collector.host_manager.dao.successful_login import SuccessfulLogin
from csle_collector.host_manager.dao.host_metrics import HostMetrics
import csle_collector.host_manager.host_manager_pb2


class HostManagerUtil:
    """
    Utility class for the host manager
    """

    @staticmethod
    def read_latest_ts_auth() -> float:
        """
        Measures the timestamp of the latest failed login attempt

        :return: the number of recently failed login attempts
        """
        try:
            cmd = constants.HOST_METRICS.LIST_FAILED_LOGIN_ATTEMPTS
            output = subprocess.run(cmd.split(" "), capture_output=True, text=True)
            login_attempts_str = output.stdout
            login_attempts = login_attempts_str.split("\n")
            login_attempts = list(filter(lambda x: x != "" and len(x) > 14, login_attempts))
            year = datetime.datetime.now().year
            parsed_ts = FailedLoginAttempt.parse_from_str(str(year) + " " +
                                                          " ".join(login_attempts[-1][0:15].split())).timestamp
            if parsed_ts is None or parsed_ts.timestamp is None:
                raise ValueError("Could not parse the login attempts log")
            return float(parsed_ts.timestamp)
        except Exception:
            return datetime.datetime.now().timestamp()

    @staticmethod
    def read_failed_login_attempts(failed_auth_last_ts: float) -> int:
        """
        Measures the number of recent failed login attempts

        :param emulation_config: configuration to connect to the node in the emulation
        :return: the number of recently failed login attempts
        """
        cmd = constants.HOST_METRICS.LIST_FAILED_LOGIN_ATTEMPTS
        output = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        login_attempts_str = output.stdout
        login_attempts = login_attempts_str.split("\n")
        login_attempts = list(filter(lambda x: x != "" and len(x) > 14, login_attempts))
        year = datetime.datetime.now().year
        failed_login_attempts: List[FailedLoginAttempt] = list(map(lambda x: FailedLoginAttempt.parse_from_str(
            str(year) + " " + " ".join(x[0:15].split())), login_attempts))
        filtered_failed_login_attempts = list(filter(lambda x: (x is not None and x.timestamp is not None
                                                                and float(x.timestamp) > failed_auth_last_ts),
                                                     failed_login_attempts))
        return len(filtered_failed_login_attempts)

    @staticmethod
    def read_latest_ts_login() -> float:
        """
        Measures the timestamp of the latest successful login attempt

        :return: the number of recently failed login attempts
        """
        try:
            cmd = constants.HOST_METRICS.LIST_SUCCESSFUL_LOGIN_ATTEMPTS
            output = subprocess.run(cmd.split(" "), capture_output=True, text=True)
            logins = output.stdout.split("\n")
            logins = list(filter(lambda x: x != "" and len(x) > 0 and "wtmp begins" not in x, logins))
            year = datetime.datetime.now().year
            ts = SuccessfulLogin.parse_from_str(" ".join(logins[0].split()), year=year).timestamp
            if ts is None:
                raise ValueError("Could not read latest login timestamp")
            return float(ts)
        except Exception:
            return datetime.datetime.now().timestamp()

    @staticmethod
    def read_successful_login_events(login_last_ts: float) -> int:
        """
        Measures the number of recent successful login attempts

        :param login_last_ts: the timestamp to use when filtering logins
        :return: the number of recently failed login attempts
        """
        cmd = constants.HOST_METRICS.LIST_SUCCESSFUL_LOGIN_ATTEMPTS
        output = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        logins = output.stdout.split("\n")
        logins = list(filter(lambda x: x != "" and len(x) > 0 and "wtmp begins" not in x, logins))
        year = datetime.datetime.now().year
        successful_logins = list(map(lambda x: SuccessfulLogin.parse_from_str(" ".join(x.split()), year=year), logins))
        successful_logins = list(filter(lambda x: x.timestamp is not None, successful_logins))
        successful_logins = list(filter(lambda x: x.timestamp > login_last_ts, successful_logins))
        return len(successful_logins)

    @staticmethod
    def read_open_connections() -> int:
        """
        Measures the number of open connections to the server

        :return: the number of open connections
        """
        cmd = constants.HOST_METRICS.LIST_OPEN_CONNECTIONS_CMD
        output = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        connections_str = output.stdout
        parts = connections_str.split("Active UNIX domain sockets", 1)
        if len(parts) > 0:
            parts2 = parts[0].split("State")
            if len(parts2) > 1:
                parts3 = parts2[1].split("\n")
                return len(parts3) - 1
        return -1

    @staticmethod
    def read_users() -> int:
        """
        Measures the number of user accounts on the server

        :return: the number of user accounts
        """
        cmd = constants.HOST_METRICS.LIST_USER_ACCOUNTS
        output = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        users_str = output.stdout
        users = users_str.split("\n")
        return len(users)

    @staticmethod
    def read_logged_in_users() -> int:
        """
        Measures the number of logged-in users on the server

        :return: the number of logged in users
        """
        cmd = constants.HOST_METRICS.LIST_LOGGED_IN_USERS_CMD
        output = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        users_str = output.stdout
        users_str = users_str.replace("\n", "")
        users = users_str.split(" ")
        return len(users)

    @staticmethod
    def read_processes() -> int:
        """
        Measures the number of processes on the server

        :param emulation_config: configuration to connect to the node in the emulation
        :return: the number of processes
        """
        try:
            cmd = constants.HOST_METRICS.LIST_NUMBER_OF_PROCESSES
            output = subprocess.run(cmd.split(" "), capture_output=True, text=True)
            processes_str = output.stdout
            num_processes = int(processes_str)
        except Exception:
            num_processes = -1
        return num_processes

    @staticmethod
    def read_host_metrics(failed_auth_last_ts: float, login_last_ts: float) -> HostMetrics:
        """
        Reads the latest metrics from the host

        :param failed_auth_last_ts: the last time-step of read failed authentication attempts
        :param login_last_ts: the last time-step of read successful logins
        :return: The parsed host metrics
        """
        num_logged_in_users = HostManagerUtil.read_logged_in_users()
        num_failed_login_attempts = HostManagerUtil.read_failed_login_attempts(
            failed_auth_last_ts=failed_auth_last_ts)
        num_open_connections = HostManagerUtil.read_open_connections()
        if num_open_connections == -1:
            num_open_connections = 0
        num_login_events = HostManagerUtil.read_successful_login_events(login_last_ts=login_last_ts)
        num_processes = HostManagerUtil.read_processes()
        if num_processes == -1:
            num_processes = 0
        num_users = HostManagerUtil.read_users()
        host_metrics = HostMetrics(
            num_logged_in_users=num_logged_in_users,
            num_failed_login_attempts=num_failed_login_attempts,
            num_open_connections=num_open_connections,
            num_login_events=num_login_events,
            num_processes=num_processes,
            num_users=num_users
        )
        return host_metrics

    @staticmethod
    def host_monitor_dto_to_dict(host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO) \
            -> Dict[str, Any]:
        """
        Converts a HostStatusDTO to a dict

        :param host_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        if isinstance(host_dto, tuple):
            host_dto = host_dto[0]
        d = {}
        d["monitor_running"] = host_dto.monitor_running
        d["filebeat_running"] = host_dto.filebeat_running
        d["packetbeat_running"] = host_dto.packetbeat_running
        d["metricbeat_running"] = host_dto.metricbeat_running
        d["heartbeat_running"] = host_dto.heartbeat_running
        return d

    @staticmethod
    def host_monitor_dto_from_dict(d: Dict[str, Any]) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Converts a dict representation of a HostStatusDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        host_dto = csle_collector.host_manager.host_manager_pb2.HostStatusDTO()
        host_dto.monitor_running = d["monitor_running"]
        host_dto.filebeat_running = d["filebeat_running"]
        host_dto.packetbeat_running = d["packetbeat_running"]
        host_dto.metricbeat_running = d["metricbeat_running"]
        host_dto.heartbeat_running = d["heartbeat_running"]
        return host_dto

    @staticmethod
    def host_metrics_dto_to_dict(host_metrics_dto: csle_collector.host_manager.host_manager_pb2.HostMetricsDTO) \
            -> Dict[str, Any]:
        """
        Converts a HostMetricsDTO to a dict

        :param host_metrics_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d: Dict[str, Any] = {}
        d["num_logged_in_users"] = host_metrics_dto.num_logged_in_users
        d["num_failed_login_attempts"] = host_metrics_dto.num_failed_login_attempts
        d["num_open_connections"] = host_metrics_dto.num_open_connections
        d["num_login_events"] = host_metrics_dto.num_login_events
        d["num_processes"] = host_metrics_dto.num_processes
        d["num_users"] = host_metrics_dto.num_users
        d["ip"] = host_metrics_dto.ip
        d["timestamp"] = host_metrics_dto.timestamp
        return d

    @staticmethod
    def host_metrics_dto_from_dict(d: Dict[str, Any]) -> csle_collector.host_manager.host_manager_pb2.HostMetricsDTO:
        """
        Converts a dict representation of a HostMetricsDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        host_metrics_dto = csle_collector.host_manager.host_manager_pb2.HostMetricsDTO()
        host_metrics_dto.num_logged_in_users = d["num_logged_in_users"]
        host_metrics_dto.num_failed_login_attempts = d["num_failed_login_attempts"]
        host_metrics_dto.num_open_connections = d["num_open_connections"]
        host_metrics_dto.num_login_events = d["num_login_events"]
        host_metrics_dto.num_processes = d["num_processes"]
        host_metrics_dto.num_users = d["num_users"]
        host_metrics_dto.ip = d["ip"]
        host_metrics_dto.timestamp = d["timestamp"]
        return host_metrics_dto

    @staticmethod
    def host_metrics_dto_empty() -> csle_collector.host_manager.host_manager_pb2.HostMetricsDTO:
        """
        :return: an empty HostMetricsDTO
        """
        host_metrics_dto = csle_collector.host_manager.host_manager_pb2.HostMetricsDTO()
        host_metrics_dto.num_logged_in_users = 0
        host_metrics_dto.num_failed_login_attempts = 0
        host_metrics_dto.num_open_connections = 0
        host_metrics_dto.num_login_events = 0
        host_metrics_dto.num_processes = 0
        host_metrics_dto.num_users = 0
        host_metrics_dto.ip = ""
        host_metrics_dto.timestamp = 0
        return host_metrics_dto

    @staticmethod
    def host_monitor_dto_empty() -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        :return: an empty HostStatusDTO
        """
        host_monitor_dto = csle_collector.host_manager.host_manager_pb2.HostStatusDTO()
        host_monitor_dto.monitor_running = False
        host_monitor_dto.filebeat_running = False
        host_monitor_dto.packetbeat_running = False
        host_monitor_dto.metricbeat_running = False
        host_monitor_dto.heartbeat_running = False
        return host_monitor_dto

    @staticmethod
    def filebeat_config(log_files_paths: List[str], kibana_ip: str, kibana_port: int, elastic_ip: str,
                        elastic_port: int, num_elastic_shards: int, kafka_topics: List[str], kafka_ip: str,
                        kafka_port: int, reload_enabled: bool = False, kafka: bool = False) \
            -> Dict[str, Any]:
        """
        Generates the filebeat.yml config

        :param log_files_paths: the list of log files that filebeat should monitor
        :param kibana_ip: the IP of Kibana where the data should be visualized
        :param kibana_port: the port of Kibana where the data should be visualized
        :param elastic_ip: the IP of elastic where the data should be shipped
        :param elastic_port: the port of elastic where the data should be shipped
        :param num_elastic_shards: the number of elastic shards
        :param reload_enabled: whether automatic reload of modules should be enabled
        :param kafka: whether kafka should be added as input
        :param kafka_topics: list of kafka topics to ingest
        :param kafka_port: the kafka server port
        :param kafka_ip: the kafka server ip
        :return: the filebeat configuration dict
        """
        filebeat_config: Dict[str, Any] = {}
        filebeat_config[constants.FILEBEAT.INPUTS_PROPERTY] = [
            {
                constants.BEATS.TYPE_PROPERTY: constants.BEATS.FILESTREAM_PROPERTY,
                constants.BEATS.ID_PROPERTY: 1,
                constants.BEATS.ENABLED_PROPERTY: True,
                constants.BEATS.PATHS_PROPERTY: log_files_paths
            }
        ]
        if kafka:
            filebeat_config[constants.FILEBEAT.INPUTS_PROPERTY].append(
                {
                    constants.BEATS.TYPE_PROPERTY: constants.BEATS.KAFKA_PROPERTY,
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.GROUP_ID_PROPERTY: constants.FILEBEAT.FILEBEAT_GROUP_ID,
                    constants.BEATS.TOPICS_PROPERTY: kafka_topics,
                    constants.BEATS.HOSTS_PROPERTY: [f"{kafka_ip}:{kafka_port}"],
                }
            )
        filebeat_config[constants.FILEBEAT.MODULES_PROPERTY] = {
            constants.BEATS.PATH_PROPERTY: f"{constants.FILEBEAT.MODULES_CONFIG_DIR}*.yml",
            constants.BEATS.RELOAD_ENABLED_PROPERTY: reload_enabled
        }
        filebeat_config[constants.BEATS.SETUP_TEMPLATE_SETTINGS_PROPERTY] = {
            constants.BEATS.INDEX_NUM_SHARDS_PROPERTY: num_elastic_shards
        }
        filebeat_config[constants.BEATS.SETUP_KIBANA_PROPERTY] = {
            constants.BEATS.HOST_PROPERTY: f"{kibana_ip}:{kibana_port}"
        }
        filebeat_config[constants.BEATS.ELASTIC_OUTPUT_PROPERTY] = {
            constants.BEATS.HOSTS_PROPERTY: [f"{elastic_ip}:{elastic_port}"]
        }
        filebeat_config[constants.BEATS.PROCESSORS_PROPERTY] = {
            constants.BEATS.ADD_HOST_METADATA_PROPERTY: {
                constants.BEATS.WHEN_NOT_CONTAIN_TAGS_PROPERTY: constants.BEATS.FORWARDED_PROPERTY
            }
        }
        return filebeat_config

    @staticmethod
    def write_yaml_config(config: Union[Dict[str, Any], List[Dict[str, Any]]], path: str) -> None:
        """
        Writes a given filebeat config to disk

        :param config: the filebeat config to write
        :param path: the path to write the file to
        :return: None
        """
        logging.info(f"Writing configuration file to path: {path}")
        with open(path, 'w') as file:
            yaml.dump(config, file)

    @staticmethod
    def filebeat_snort_module_config() -> List[Dict[str, Any]]:
        """
        :return: the snort filebeat module config
        """
        snort_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.FILEBEAT.SNORT_MODULE,
                constants.BEATS.LOG_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.VAR_INPUT_PROPERTY: constants.BEATS.FILE_PROPERTY,
                    constants.BEATS.VAR_PATHS_PROPERTY: [
                        constants.SNORT_IDS_ROUTER.SNORT_FAST_LOG_FILE
                    ]
                }
            }
        ]
        return snort_config

    @staticmethod
    def filebeat_elasticsearch_module_config() -> List[Dict[str, Any]]:
        """
        :return: the elasticsearch filebeat module config
        """
        elastic_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.FILEBEAT.ELASTICSEARCH_MODULE,
                constants.BEATS.SERVER_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.VAR_PATHS_PROPERTY: [
                        f"{constants.ELK.ELASTICSEARCH_LOG_DIR}*.log",
                        f"{constants.ELK.ELASTICSEARCH_LOG_DIR}*_server.json"
                    ]
                }
            }
        ]
        return elastic_config

    @staticmethod
    def filebeat_logstash_module_config() -> List[Dict[str, Any]]:
        """
        :return: the logstash filebeat module config
        """
        logstash_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.FILEBEAT.LOGSTASH_MODULE,
                constants.BEATS.LOG_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.VAR_PATHS_PROPERTY: [
                        f"{constants.ELK.LOGSTASH_LOG_DIR}logstash.log*"
                    ]
                },
                constants.BEATS.SLOWLOG_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.VAR_PATHS_PROPERTY: [
                        f"{constants.ELK.LOGSTASH_LOG_DIR}logstash-slowlog.log*"
                    ]
                }
            }
        ]
        return logstash_config

    @staticmethod
    def filebeat_kibana_module_config() -> List[Dict[str, Any]]:
        """
        :return: the kibana filebeat module config
        """
        kibana_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.FILEBEAT.KIBANA_MODULE,
                constants.BEATS.LOG_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.VAR_PATHS_PROPERTY: [
                        f"{constants.ELK.KIBANA_LOG_DIR}*.log"
                    ]
                },
                constants.BEATS.AUDIT_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True
                }
            }
        ]
        return kibana_config

    @staticmethod
    def filebeat_system_module_config() -> List[Dict[str, Any]]:
        """
        :return: the system filebeat module config
        """
        system_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.FILEBEAT.SYSTEM_MODULE,
                constants.BEATS.SYSLOG_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.VAR_PATHS_PROPERTY: [
                        f"{constants.SYSTEM.SYSLOG}*"
                    ]
                },
                constants.BEATS.AUTH_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.VAR_PATHS_PROPERTY: [
                        f"{constants.SYSTEM.AUTH_LOG}"
                    ]
                }
            }
        ]
        return system_config

    @staticmethod
    def filebeat_kafka_module_config() -> List[Dict[str, Any]]:
        """
        :return: the kafka filebeat module config
        """
        system_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.FILEBEAT.KAFKA_MODULE,
                constants.BEATS.LOG_PROPERTY: {
                    constants.BEATS.ENABLED_PROPERTY: True,
                    constants.BEATS.VAR_PATHS_PROPERTY: [
                        f"{constants.KAFKA.DIR}controller.log*",
                        f"{constants.KAFKA.DIR}server.log*",
                        f"{constants.KAFKA.DIR}state-change.log*",
                        f"{constants.KAFKA.DIR}kafka-*.log*"
                    ]
                }
            }
        ]
        return system_config

    @staticmethod
    def packetbeat_config(kibana_ip: str, kibana_port: int, elastic_ip: str, elastic_port: int,
                          num_elastic_shards: int) -> Dict[str, Any]:
        """
        Generates the packetbeat.yml config

        :param kibana_ip: the IP of Kibana where the data should be visualized
        :param kibana_port: the port of Kibana where the data should be visualized
        :param elastic_ip: the IP of elastic where the data should be shipped
        :param elastic_port: the port of elastic where the data should be shipped
        :param num_elastic_shards: the number of elastic shards
        :return: the filebeat configuration dict
        """
        packetbeat_config: Dict[str, Any] = {}
        packetbeat_config[constants.PACKETBEAT.INTERFACES_TYPE_PROPERTY] = constants.PACKETBEAT.AF_PACKET_PROPERTY
        packetbeat_config[constants.PACKETBEAT.INTERFACES_DEVICE_PROPERTY] = constants.PACKETBEAT.ANY_DEVICE_PROPERTY
        packetbeat_config[constants.PACKETBEAT.FLOWS] = {
            constants.PACKETBEAT.TIMEOUT_PROPERTY: "30s",
            constants.BEATS.PERIOD_PROPERTY: "10s"
        }
        packetbeat_config[constants.PACKETBEAT.PROTOCOLS] = [
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.ICMP_PROTOCOL,
                constants.BEATS.ENABLED_PROPERTY: True
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.AMQP_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.AMQP_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.CASSANDRA_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.CASSANDRA_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.DHCPV4_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.DHCPV4_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.DNS_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.DNS_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.HTTP_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.HTTP_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.MEMCACHE_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.MEMCACHE_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.MYSQL_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.MYSQL_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.PGSQL_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.PGSQL_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.REDIS_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.REDIS_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.THRIFT_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.THRIFT_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.MONGODB_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.MONGODB_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.NFS_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.NFS_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.TLS_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.TLS_PORTS
            },
            {
                constants.BEATS.TYPE_PROPERTY: constants.PACKETBEAT.SIP_PROTOCOL,
                constants.PACKETBEAT.PORTS_PROPERTY: constants.PACKETBEAT.SIP_PORTS
            }
        ]
        packetbeat_config[constants.BEATS.SETUP_TEMPLATE_SETTINGS_PROPERTY] = {
            constants.BEATS.INDEX_NUM_SHARDS_PROPERTY: num_elastic_shards
        }
        packetbeat_config[constants.BEATS.SETUP_KIBANA_PROPERTY] = {
            constants.BEATS.HOST_PROPERTY: f"{kibana_ip}:{kibana_port}"
        }
        packetbeat_config[constants.BEATS.ELASTIC_OUTPUT_PROPERTY] = {
            constants.BEATS.HOSTS_PROPERTY: [f"{elastic_ip}:{elastic_port}"]
        }
        packetbeat_config[constants.BEATS.PROCESSORS_PROPERTY] = {
            constants.BEATS.ADD_HOST_METADATA_PROPERTY: {
                constants.BEATS.WHEN_NOT_CONTAIN_TAGS_PROPERTY: constants.BEATS.FORWARDED_PROPERTY
            }
        }
        return packetbeat_config

    @staticmethod
    def metricbeat_config(kibana_ip: str, kibana_port: int, elastic_ip: str,
                          elastic_port: int, num_elastic_shards: int, reload_enabled: bool = False) -> Dict[str, Any]:
        """
        Generates the metricbeat.yml config

        :param kibana_ip: the IP of Kibana where the data should be visualized
        :param kibana_port: the port of Kibana where the data should be visualized
        :param elastic_ip: the IP of elastic where the data should be shipped
        :param elastic_port: the port of elastic where the data should be shipped
        :param num_elastic_shards: the number of elastic shards
        :param reload_enabled: whether automatic reload of modules should be enabled
        :return: the metricbeat configuration dict
        """
        metricbeat_config = {}
        metricbeat_config[constants.METRICBEAT.MODULES_PROPERTY] = {
            constants.BEATS.PATH_PROPERTY: f"{constants.METRICBEAT.MODULES_CONFIG_DIR}*.yml",
            constants.BEATS.RELOAD_ENABLED_PROPERTY: reload_enabled
        }
        metricbeat_config[constants.BEATS.SETUP_TEMPLATE_SETTINGS_PROPERTY] = {
            constants.BEATS.INDEX_NUM_SHARDS_PROPERTY: num_elastic_shards
        }
        metricbeat_config[constants.BEATS.SETUP_KIBANA_PROPERTY] = {
            constants.BEATS.HOST_PROPERTY: f"{kibana_ip}:{kibana_port}"
        }
        metricbeat_config[constants.BEATS.ELASTIC_OUTPUT_PROPERTY] = {
            constants.BEATS.HOSTS_PROPERTY: [f"{elastic_ip}:{elastic_port}"]
        }
        metricbeat_config[constants.BEATS.PROCESSORS_PROPERTY] = {
            constants.BEATS.ADD_HOST_METADATA_PROPERTY: {
                constants.BEATS.WHEN_NOT_CONTAIN_TAGS_PROPERTY: constants.BEATS.FORWARDED_PROPERTY
            }
        }
        return metricbeat_config

    @staticmethod
    def metricbeat_elasticsearch_module_config(elastic_ip: str, elastic_port: int) -> List[Dict[str, Any]]:
        """
        :return: the elasticsearch metricbeat module config
        """
        elastic_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.FILEBEAT.ELASTICSEARCH_MODULE,
                constants.BEATS.PERIOD_PROPERTY: "10s",
                constants.BEATS.HOSTS_PROPERTY: [f"{elastic_ip}:{elastic_port}"]
            }
        ]
        return elastic_config

    @staticmethod
    def metricbeat_kibana_module_config(kibana_ip: str, kibana_port: int) -> List[Dict[str, Any]]:
        """
        Creates the kibana metricbeat module configuration

        :param kibana_ip: the kibana host IP
        :param kibana_port: the kibana host port
        :return: the kibana metricbeat module config
        """
        kibana_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.METRICBEAT.KIBANA_MODULE,
                constants.BEATS.PERIOD_PROPERTY: "10s",
                constants.BEATS.HOSTS_PROPERTY: [f"{kibana_ip}:{kibana_port}"]
            }
        ]
        return kibana_config

    @staticmethod
    def metricbeat_logstash_module_config(logstash_ip: str, logstash_port: int) -> List[Dict[str, Any]]:
        """
        Creates the logstash metricbeat module configuration

        :param logstash_ip: the kibana host IP
        :param logstash_port: the kibana host port
        :return: the kibana metricbeat module config
        """
        logstash_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.METRICBEAT.LOGSTASH_MODULE,
                constants.BEATS.PERIOD_PROPERTY: "10s",
                constants.BEATS.HOSTS_PROPERTY: [f"{logstash_ip}:{logstash_port}"]
            }
        ]
        return logstash_config

    @staticmethod
    def metricbeat_kafka_module_config(kafka_ip: str, kafka_port: int) -> List[Dict[str, Any]]:
        """
        Creates the kafka metricbeat module configuration

        :param kafka_ip: the kibana host IP
        :param kafka_port: the kibana host port
        :return: the kafka metricbeat module config
        """
        kafka_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.METRICBEAT.KAFKA_MODULE,
                constants.BEATS.PERIOD_PROPERTY: "10s",
                constants.BEATS.HOSTS_PROPERTY: [f"{kafka_ip}:{kafka_port}"]
            }
        ]
        return kafka_config

    @staticmethod
    def metricbeat_system_module_config() -> List[Dict[str, Any]]:
        """
        Creates the system metricbeat module configuration

        :return: the system metricbeat module config
        """
        system_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.METRICBEAT.SYSTEM_MODULE,
                constants.BEATS.PERIOD_PROPERTY: "10s",
                constants.BEATS.METRICSETS_PROPERTY: [
                    constants.METRICBEAT.CPU_METRIC,
                    constants.METRICBEAT.LOAD_METRIC,
                    constants.METRICBEAT.MEMORY_METRIC,
                    constants.METRICBEAT.NETWORK_METRIC,
                    constants.METRICBEAT.PROCESS_METRIC,
                    constants.METRICBEAT.PROCESS_SUMMARY_METRIC,
                    constants.METRICBEAT.SOCKET_SUMMARY_METRIC
                ],
                constants.BEATS.ENABLED_PROPERTY: True,
                constants.METRICBEAT.PROCESSES_PROPERTY: [".*"],
                constants.METRICBEAT.CPU_METRICS_PROPERTY: [constants.METRICBEAT.PERCENTAGES_PROPERTY,
                                                            constants.METRICBEAT.NORMALIZED_PERCENTAGES_PROPERTY],
                constants.METRICBEAT.CORE_METRICS_PROPERTY: [constants.METRICBEAT.PERCENTAGES_PROPERTY]
            }
        ]
        return system_config

    @staticmethod
    def metricbeat_linux_module_config() -> List[Dict[str, Any]]:
        """
        Creates the linux metricbeat module configuration

        :return: the linux metricbeat module config
        """
        linux_config = [
            {
                constants.BEATS.MODULE_PROPERTY: constants.METRICBEAT.LINUX_MODULE,
                constants.BEATS.PERIOD_PROPERTY: "10s",
                constants.BEATS.METRICSETS_PROPERTY: {
                    constants.METRICBEAT.PAGEINFO_METRIC,
                    constants.METRICBEAT.SUMMARY_METRIC
                },
                constants.BEATS.ENABLED_PROPERTY: True
            }
        ]
        return linux_config

    @staticmethod
    def heartbeat_config(kibana_ip: str, kibana_port: int, elastic_ip: str, elastic_port: int,
                         num_elastic_shards: int, hosts_to_monitor: List[str]) -> Dict[str, Any]:
        """
        Generates the heartbeat.yml config

        :param kibana_ip: the IP of Kibana where the data should be visualized
        :param kibana_port: the port of Kibana where the data should be visualized
        :param elastic_ip: the IP of elastic where the data should be shipped
        :param elastic_port: the port of elastic where the data should be shipped
        :param num_elastic_shards: the number of elastic shards
        :param hosts_to_monitor: a list of host IP addresses to monitor
        :return: the heartbeat configuration dict
        """
        heartbeat_config: Dict[str, Any] = {}
        heartbeat_config[constants.HEARTBEAT.HEARTBEAT_MONITORS_PROPERTY] = [
            {
                constants.BEATS.TYPE_PROPERTY: constants.HEARTBEAT.ICMP_MONITOR_TYPE,
                constants.HEARTBEAT.SCHEDULE_PROPERTY: '*/5 * * * * * *',
                constants.BEATS.HOSTS_PROPERTY: hosts_to_monitor,
                constants.BEATS.ID_PROPERTY: constants.HEARTBEAT.CSLE_MONITOR_SERVICE_ID,
                constants.BEATS.NAME_PROPERTY: constants.HEARTBEAT.CSLE_MONITOR_SERVICE_NAME
            }
        ]
        heartbeat_config[constants.BEATS.SETUP_TEMPLATE_SETTINGS_PROPERTY] = {
            constants.BEATS.INDEX_NUM_SHARDS_PROPERTY: num_elastic_shards
        }
        heartbeat_config[constants.BEATS.SETUP_KIBANA_PROPERTY] = {
            constants.BEATS.HOST_PROPERTY: f"{kibana_ip}:{kibana_port}"
        }
        heartbeat_config[constants.BEATS.ELASTIC_OUTPUT_PROPERTY] = {
            constants.BEATS.HOSTS_PROPERTY: [f"{elastic_ip}:{elastic_port}"]
        }
        heartbeat_config[constants.BEATS.PROCESSORS_PROPERTY] = {
            constants.BEATS.ADD_HOST_METADATA_PROPERTY: {
                constants.BEATS.WHEN_NOT_CONTAIN_TAGS_PROPERTY: constants.BEATS.FORWARDED_PROPERTY
            }
        }
        return heartbeat_config



from typing import List, Dict, Any, Tuple
import time
import numpy as np
import csle_collector.constants.constants as constants
from csle_collector.ossec_ids_manager.dao.ossec_ids_alert import OSSECIDSAlert
import csle_collector.ossec_ids_manager.ossec_ids_manager_pb2
from csle_base.grpc_serializable import GRPCSerializable
from csle_base.kafka_serializable import KafkaSerializable
from csle_base.json_serializable import JSONSerializable


class OSSECIdsAlertCounters(GRPCSerializable, KafkaSerializable, JSONSerializable):
    """
    DTO containing statistics from the OSSEC log
    """
    def __init__(self) -> None:
        """
        Initializes the DTO
        """
        self.level_alerts = list(np.zeros(16))
        self.group_alerts = list(np.zeros(len(set(constants.OSSEC.OSSEC_IDS_ALERT_GROUP_ID.values()))))
        self.severe_alerts = 0
        self.warning_alerts = 0
        self.total_alerts = 0
        self.alerts_weighted_by_level = 0
        self.ip = ""
        self.ts = 0.0

    def add(self, alert_counters: "OSSECIdsAlertCounters") -> None:
        """
        Adds another alert counters object to this one

        :param alert_counters: the counters to add
        :return: None
        """
        self.severe_alerts = self.severe_alerts + alert_counters.severe_alerts
        self.warning_alerts = self.warning_alerts + alert_counters.warning_alerts
        self.total_alerts = self.total_alerts + alert_counters.total_alerts
        self.alerts_weighted_by_level = self.alerts_weighted_by_level + alert_counters.alerts_weighted_by_level
        for idx in range(len(self.level_alerts)):
            self.level_alerts[idx] = self.level_alerts[idx] + alert_counters.level_alerts[idx]
        for idx in range(len(self.group_alerts)):
            self.group_alerts[idx] = self.group_alerts[idx] + alert_counters.group_alerts[idx]

    def count(self, alerts: List[OSSECIDSAlert]) -> None:
        """
        Counts the list of alerts

        :param alerts: list of alerts from the log
        :return: None
        """
        for a in alerts:
            if a.level in range(0, len(self.level_alerts)):
                self.level_alerts[a.level] += 1
            for group_id in a.group_ids:
                if group_id in range(0, len(self.group_alerts)):
                    self.group_alerts[group_id] += 1

        self.total_alerts = len(alerts)
        self.severe_alerts = sum(self.level_alerts[0:constants.OSSEC.OSSEC_SEVERE_ALERT_LEVEL_THRESHOLD])
        self.warning_alerts = sum(self.level_alerts[constants.OSSEC.OSSEC_SEVERE_ALERT_LEVEL_THRESHOLD:])
        self.alerts_weighted_by_level = 0
        for idx in range(len(self.level_alerts)):
            level = idx
            self.alerts_weighted_by_level += level * self.level_alerts[idx]

    @staticmethod
    def from_kafka_record(record: str) -> "OSSECIdsAlertCounters":
        """
        Converts a kafka record to a DTO

        :param record: the kafka record to convert
        :return: the DTO
        """
        parts = record.split(",")
        obj = OSSECIdsAlertCounters()
        obj.ts = float(parts[0])
        obj.ip = parts[1]
        obj.total_alerts = int(round(float(parts[2])))
        obj.warning_alerts = int(round(float(parts[3])))
        obj.severe_alerts = int(round(float(parts[4])))
        obj.alerts_weighted_by_level = int(round(float(parts[5])))
        obj.group_alerts = []
        obj.level_alerts = []
        for i in range(6, len(set(constants.OSSEC.OSSEC_ALERT_RULE_ID_TO_DESCR.keys())) + 6):
            obj.level_alerts.append(int(round(float(parts[i]))))
        for i in range(len(set(constants.OSSEC.OSSEC_ALERT_RULE_ID_TO_DESCR.keys())) + 6,
                       len(set(constants.OSSEC.OSSEC_IDS_ALERT_GROUP_ID.values())) +
                       len(set(constants.OSSEC.OSSEC_ALERT_RULE_ID_TO_DESCR.keys())) + 6):
            obj.group_alerts.append(int(round(float(parts[i]))))
        return obj

    def update_with_kafka_record(self, record: str) -> None:
        """
        Updates the DTO with a kafka record

        :param record: the kafka record to use for updating
        :return: None
        """
        parts = record.split(",")
        self.ts = float(parts[0])
        self.ip = parts[1]
        self.total_alerts = int(round(float(parts[2])))
        self.warning_alerts = int(round(float(parts[3])))
        self.severe_alerts = int(round(float(parts[4])))
        self.alerts_weighted_by_level = int(round(float(parts[5])))

        self.group_alerts = []
        self.level_alerts = []
        for i in range(6, len(set(constants.OSSEC.OSSEC_ALERT_RULE_ID_TO_DESCR.keys())) + 6):
            self.level_alerts.append(int(round(float(parts[i]))))
        for i in range(len(set(constants.OSSEC.OSSEC_ALERT_RULE_ID_TO_DESCR.keys())) + 6,
                       len(set(constants.OSSEC.OSSEC_IDS_ALERT_GROUP_ID.values()))
                       + len(set(constants.OSSEC.OSSEC_ALERT_RULE_ID_TO_DESCR.keys())) + 6):
            self.group_alerts.append(int(round(float(parts[i]))))

    def to_kafka_record(self, ip: str) -> str:
        """
        Converts the DTO into a kafka record

        :param ip: the ip to add to the record in addition to the IDS statistics
        :return: a comma-separated string representing the kafka record
        """
        ts = time.time()
        total_counters = [ts, ip, self.total_alerts, self.warning_alerts, self.severe_alerts,
                          self.alerts_weighted_by_level] + self.group_alerts + self.level_alerts
        record_str = ",".join(list(map(lambda x: str(x), total_counters)))
        return record_str

    @staticmethod
    def from_grpc_object(obj: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO) \
            -> "OSSECIdsAlertCounters":
        """
        Instantiate the object from a GRPC object

        :param obj: the object to instantiate from
        :return: the instantiated grpc object
        """
        instance = OSSECIdsAlertCounters()
        instance.ts = time.time()
        instance.ip = obj.ip
        instance.total_alerts = obj.total_alerts
        instance.warning_alerts = obj.warning_alerts
        instance.severe_alerts = obj.severe_alerts
        instance.alerts_weighted_by_level = obj.alerts_weighted_by_level
        instance.level_alerts[0] = obj.level_0_alerts
        instance.level_alerts[1] = obj.level_1_alerts
        instance.level_alerts[2] = obj.level_2_alerts
        instance.level_alerts[3] = obj.level_3_alerts
        instance.level_alerts[4] = obj.level_4_alerts
        instance.level_alerts[5] = obj.level_5_alerts
        instance.level_alerts[6] = obj.level_6_alerts
        instance.level_alerts[7] = obj.level_7_alerts
        instance.level_alerts[8] = obj.level_8_alerts
        instance.level_alerts[9] = obj.level_9_alerts
        instance.level_alerts[10] = obj.level_10_alerts
        instance.level_alerts[11] = obj.level_11_alerts
        instance.level_alerts[12] = obj.level_12_alerts
        instance.level_alerts[13] = obj.level_13_alerts
        instance.level_alerts[14] = obj.level_14_alerts
        instance.level_alerts[15] = obj.level_15_alerts
        instance.group_alerts[0] = obj.invalid_login_alerts
        instance.group_alerts[1] = obj.authentication_success_alerts
        instance.group_alerts[2] = obj.authentication_failed_alerts
        instance.group_alerts[3] = obj.connection_attempt_alerts
        instance.group_alerts[4] = obj.attacks_alerts
        instance.group_alerts[5] = obj.adduser_alerts
        instance.group_alerts[6] = obj.sshd_alerts
        instance.group_alerts[7] = obj.ids_alerts
        instance.group_alerts[8] = obj.firewall_alerts
        instance.group_alerts[9] = obj.squid_alerts
        instance.group_alerts[10] = obj.apache_alerts
        instance.group_alerts[11] = obj.syslog_alerts
        return instance

    def to_grpc_object(self) -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO:
        """
        Converts the object into a gRPC DTO for serialization

        :return: A csle_collector.snort_ids_manager.snort_ids_manager_pb2.IdsLogDTOb
        """
        ts = time.time()
        return csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO(
            timestamp=float(ts),
            ip=self.ip,
            total_alerts=int(self.total_alerts),
            warning_alerts=int(self.warning_alerts),
            severe_alerts=int(self.severe_alerts),
            alerts_weighted_by_level=int(self.alerts_weighted_by_level),
            level_0_alerts=int(self.level_alerts[0]),
            level_1_alerts=int(self.level_alerts[1]),
            level_2_alerts=int(self.level_alerts[2]),
            level_3_alerts=int(self.level_alerts[3]),
            level_4_alerts=int(self.level_alerts[4]),
            level_5_alerts=int(self.level_alerts[5]),
            level_6_alerts=int(self.level_alerts[6]),
            level_7_alerts=int(self.level_alerts[7]),
            level_8_alerts=int(self.level_alerts[8]),
            level_9_alerts=int(self.level_alerts[9]),
            level_10_alerts=int(self.level_alerts[10]),
            level_11_alerts=int(self.level_alerts[11]),
            level_12_alerts=int(self.level_alerts[12]),
            level_13_alerts=int(self.level_alerts[13]),
            level_14_alerts=int(self.level_alerts[14]),
            level_15_alerts=int(self.level_alerts[15]),
            invalid_login_alerts=int(self.group_alerts[0]),
            authentication_success_alerts=int(self.group_alerts[1]),
            authentication_failed_alerts=int(self.group_alerts[2]),
            connection_attempt_alerts=int(self.group_alerts[3]),
            attacks_alerts=int(self.group_alerts[4]),
            adduser_alerts=int(self.group_alerts[5]),
            sshd_alerts=int(self.group_alerts[6]),
            ids_alerts=int(self.group_alerts[7]),
            firewall_alerts=int(self.group_alerts[8]),
            squid_alerts=int(self.group_alerts[9]),
            apache_alerts=int(self.group_alerts[10]),
            syslog_alerts=int(self.group_alerts[11])
        )

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"total_alerts: {self.total_alerts}, warning_alerts: {self.warning_alerts}, " \
               f"severe_alerts: {self.severe_alerts}, " \
               f"group_alerts: {self.level_alerts}, group_alerts: {self.group_alerts}," \
               f"alerts_weighted_by_level: {self.alerts_weighted_by_level}"

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "OSSECIdsAlertCounters":
        """
        Converts a dict representation of the object into an instance

        :param d: the dict to convert
        :return: the DTO
        """
        obj = OSSECIdsAlertCounters()
        obj.ip = d["ip"]
        obj.ts = d["ts"]
        obj.total_alerts = d["total_alerts"]
        obj.warning_alerts = d["warning_alerts"]
        obj.severe_alerts = d["severe_alerts"]
        obj.level_alerts = d["level_alerts"]
        obj.group_alerts = d["group_alerts"]
        obj.alerts_weighted_by_level = d["alerts_weighted_by_level"]
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Any] = {}
        d["ip"] = self.ip
        d["ts"] = self.ts
        d["group_alerts"] = list(self.group_alerts)
        d["level_alerts"] = list(self.level_alerts)
        d["total_alerts"] = self.total_alerts
        d["warning_alerts"] = self.warning_alerts
        d["severe_alerts"] = self.severe_alerts
        d["alerts_weighted_by_level"] = self.alerts_weighted_by_level
        return d

    def copy(self) -> "OSSECIdsAlertCounters":
        """
        :return: a copy of the object
        """
        c = OSSECIdsAlertCounters()
        c.group_alerts = self.group_alerts
        c.level_alerts = self.level_alerts
        c.ip = self.ip
        c.ts = self.ts
        c.total_alerts = self.total_alerts
        c.warning_alerts = self.warning_alerts
        c.severe_alerts = self.severe_alerts
        c.alerts_weighted_by_level = self.alerts_weighted_by_level
        return c

    def get_deltas(self, counters_prime: "OSSECIdsAlertCounters") -> Tuple[List[int], List[str]]:
        """
        Get the deltas between two counters objects

        :param counters_prime: the counters object to compare with
        :return: the deltas and the labels
        """
        deltas_level = list(np.array(counters_prime.level_alerts).astype(int).tolist())
        deltas_group = list(np.array(counters_prime.group_alerts).astype(int).tolist())
        deltas = ([int(counters_prime.total_alerts), int(counters_prime.warning_alerts),
                  int(counters_prime.severe_alerts), int(counters_prime.alerts_weighted_by_level)] +
                  deltas_level + deltas_group)
        labels = constants.KAFKA_CONFIG.OSSEC_IDS_ALERTS_LABELS
        assert len(labels) == len(deltas)
        return list(deltas), labels

    def num_attributes(self) -> int:
        """
        :return: The number of attributes of the DTO
        """
        return 22 + len(set(constants.OSSEC.OSSEC_IDS_ALERT_GROUP_ID.values()))

    @staticmethod
    def schema() -> "OSSECIdsAlertCounters":
        """
        :return: get the schema of the DTO
        """
        return OSSECIdsAlertCounters()

    @staticmethod
    def from_json_file(json_file_path: str) -> "OSSECIdsAlertCounters":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return OSSECIdsAlertCounters.from_dict(json.loads(json_str))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: host_manager.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12host_manager.proto\"\x0e\n\x0cStopSparkMsg\"\x0f\n\rStartSparkMsg\"\x14\n\x12StopHostMonitorMsg\"Z\n\x13StartHostMonitorMsg\x12\x10\n\x08kafka_ip\x18\x01 \x01(\t\x12\x12\n\nkafka_port\x18\x02 \x01(\x05\x12\x1d\n\x15time_step_len_seconds\x18\x04 \x01(\x05\"\x11\n\x0fStopFilebeatMsg\"\x12\n\x10StartFilebeatMsg\"\x97\x02\n\x11\x43onfigFilebeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x16\n\x0ereload_enabled\x18\x06 \x01(\x08\x12\x10\n\x08kafka_ip\x18\x07 \x01(\t\x12\x12\n\nkafka_port\x18\x08 \x01(\x05\x12\r\n\x05kafka\x18\t \x01(\x08\x12\x14\n\x0ckafka_topics\x18\n \x03(\t\x12\x18\n\x10\x66ilebeat_modules\x18\x0b \x03(\t\x12\x17\n\x0flog_files_paths\x18\x0c \x03(\t\"\x13\n\x11StopPacketbeatMsg\"\x14\n\x12StartPacketbeatMsg\"\x83\x01\n\x13\x43onfigPacketbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\"\x13\n\x11StopMetricbeatMsg\"\x14\n\x12StartMetricbeatMsg\"\xdd\x01\n\x13\x43onfigMetricbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x10\n\x08kafka_ip\x18\x06 \x01(\t\x12\x12\n\nkafka_port\x18\x07 \x01(\x05\x12\x16\n\x0ereload_enabled\x18\x08 \x01(\x08\x12\x1a\n\x12metricbeat_modules\x18\x0c \x03(\t\"\x12\n\x10GetHostStatusMsg\"\x95\x01\n\rHostStatusDTO\x12\x17\n\x0fmonitor_running\x18\x01 \x01(\x08\x12\x18\n\x10\x66ilebeat_running\x18\x02 \x01(\x08\x12\x1a\n\x12packetbeat_running\x18\x03 \x01(\x08\x12\x1a\n\x12metricbeat_running\x18\x04 \x01(\x08\x12\x19\n\x11heartbeat_running\x18\x05 \x01(\x08\"G\n\x11GetHostMetricsMsg\x12\x1b\n\x13\x66\x61iled_auth_last_ts\x18\x01 \x01(\x02\x12\x15\n\rlogin_last_ts\x18\x02 \x01(\x02\"\xd1\x01\n\x0eHostMetricsDTO\x12\x1b\n\x13num_logged_in_users\x18\x01 \x01(\x05\x12!\n\x19num_failed_login_attempts\x18\x02 \x01(\x05\x12\x1c\n\x14num_open_connections\x18\x03 \x01(\x05\x12\x18\n\x10num_login_events\x18\x04 \x01(\x05\x12\x15\n\rnum_processes\x18\x05 \x01(\x05\x12\x11\n\tnum_users\x18\x06 \x01(\x05\x12\n\n\x02ip\x18\x07 \x01(\t\x12\x11\n\ttimestamp\x18\x08 \x01(\x02\"\x12\n\x10StopHeartbeatMsg\"\x13\n\x11StartHeartbeatMsg\"\x9c\x01\n\x12\x43onfigHeartbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x18\n\x10hosts_to_monitor\x18\x06 \x03(\t2\xf6\x07\n\x0bHostManager\x12\x38\n\x0fstopHostMonitor\x12\x13.StopHostMonitorMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10startHostMonitor\x12\x14.StartHostMonitorMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rgetHostStatus\x12\x11.GetHostStatusMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x37\n\x0egetHostMetrics\x12\x12.GetHostMetricsMsg\x1a\x0f.HostMetricsDTO\"\x00\x12\x32\n\x0cstopFilebeat\x12\x10.StopFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rstartFilebeat\x12\x11.StartFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0e\x63onfigFilebeat\x12\x12.ConfigFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estopPacketbeat\x12\x12.StopPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0fstartPacketbeat\x12\x13.StartPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10\x63onfigPacketbeat\x12\x14.ConfigPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estopMetricbeat\x12\x12.StopMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0fstartMetricbeat\x12\x13.StartMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10\x63onfigMetricbeat\x12\x14.ConfigMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rstopHeartbeat\x12\x11.StopHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estartHeartbeat\x12\x12.StartHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0f\x63onfigHeartbeat\x12\x13.ConfigHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12,\n\tstopSpark\x12\r.StopSparkMsg\x1a\x0e.HostStatusDTO\"\x00\x12.\n\nstartSpark\x12\x0e.StartSparkMsg\x1a\x0e.HostStatusDTO\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'host_manager_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPSPARKMSG._serialized_start=22
  _STOPSPARKMSG._serialized_end=36
  _STARTSPARKMSG._serialized_start=38
  _STARTSPARKMSG._serialized_end=53
  _STOPHOSTMONITORMSG._serialized_start=55
  _STOPHOSTMONITORMSG._serialized_end=75
  _STARTHOSTMONITORMSG._serialized_start=77
  _STARTHOSTMONITORMSG._serialized_end=167
  _STOPFILEBEATMSG._serialized_start=169
  _STOPFILEBEATMSG._serialized_end=186
  _STARTFILEBEATMSG._serialized_start=188
  _STARTFILEBEATMSG._serialized_end=206
  _CONFIGFILEBEATMSG._serialized_start=209
  _CONFIGFILEBEATMSG._serialized_end=488
  _STOPPACKETBEATMSG._serialized_start=490
  _STOPPACKETBEATMSG._serialized_end=509
  _STARTPACKETBEATMSG._serialized_start=511
  _STARTPACKETBEATMSG._serialized_end=531
  _CONFIGPACKETBEATMSG._serialized_start=534
  _CONFIGPACKETBEATMSG._serialized_end=665
  _STOPMETRICBEATMSG._serialized_start=667
  _STOPMETRICBEATMSG._serialized_end=686
  _STARTMETRICBEATMSG._serialized_start=688
  _STARTMETRICBEATMSG._serialized_end=708
  _CONFIGMETRICBEATMSG._serialized_start=711
  _CONFIGMETRICBEATMSG._serialized_end=932
  _GETHOSTSTATUSMSG._serialized_start=934
  _GETHOSTSTATUSMSG._serialized_end=952
  _HOSTSTATUSDTO._serialized_start=955
  _HOSTSTATUSDTO._serialized_end=1104
  _GETHOSTMETRICSMSG._serialized_start=1106
  _GETHOSTMETRICSMSG._serialized_end=1177
  _HOSTMETRICSDTO._serialized_start=1180
  _HOSTMETRICSDTO._serialized_end=1389
  _STOPHEARTBEATMSG._serialized_start=1391
  _STOPHEARTBEATMSG._serialized_end=1409
  _STARTHEARTBEATMSG._serialized_start=1411
  _STARTHEARTBEATMSG._serialized_end=1430
  _CONFIGHEARTBEATMSG._serialized_start=1433
  _CONFIGHEARTBEATMSG._serialized_end=1589
  _HOSTMANAGER._serialized_start=1592
  _HOSTMANAGER._serialized_end=2606
# @@protoc_insertion_point(module_scope)



from typing import Dict, Any
from csle_collector.client_manager.dao.arrival_config import ArrivalConfig
from csle_collector.client_manager.dao.client_arrival_type import ClientArrivalType
import csle_collector.client_manager.client_manager_pb2


class SineArrivalConfig(ArrivalConfig):
    """
    DTO representing the configuration of a sine-modulated poisson arrival process with exponential service times
    """

    def __init__(self, lamb: float, time_scaling_factor: float, period_scaling_factor: float):
        """
        Initializes the object

        :param lamb: the static arrival rate
        :param time_scaling_factor: the time-scaling factor for sine-modulated arrival processes
        :param period_scaling_factor: the period-scaling factor for sine-modulated arrival processes
        """
        self.lamb = lamb
        self.time_scaling_factor = time_scaling_factor
        self.period_scaling_factor = period_scaling_factor
        super(SineArrivalConfig, self).__init__(client_arrival_type=ClientArrivalType.SINE_MODULATED)

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"Arrival type: Sine-modulated Poisson process, " \
               f"lamb: {self.lamb}, time_scaling_factor: {self.time_scaling_factor}, " \
               f"period_scaling_factor: {self.period_scaling_factor}, client_arrival_type: {self.client_arrival_type}"

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d = {}
        d["lamb"] = self.lamb
        d["time_scaling_factor"] = self.time_scaling_factor
        d["period_scaling_factor"] = self.period_scaling_factor
        d["client_arrival_type"] = self.client_arrival_type
        return d

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "SineArrivalConfig":
        """
        Converts a dict representation of the object to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = SineArrivalConfig(lamb=d["lamb"], time_scaling_factor=d["time_scaling_factor"],
                                period_scaling_factor=d["period_scaling_factor"])
        return obj

    def to_grpc_object(self) -> csle_collector.client_manager.client_manager_pb2.SineArrivalConfigDTO:
        """
        :return: a GRPC serializable version of the object
        """
        return csle_collector.client_manager.client_manager_pb2.SineArrivalConfigDTO(
            lamb=self.lamb, time_scaling_factor=self.time_scaling_factor,
            period_scaling_factor=self.period_scaling_factor
        )

    @staticmethod
    def from_grpc_object(obj: csle_collector.client_manager.client_manager_pb2.SineArrivalConfigDTO) \
            -> "SineArrivalConfig":
        """
        Instantiates the object from a GRPC DTO

        :param obj: the object to instantiate from
        :return: the instantiated object
        """
        return SineArrivalConfig(lamb=obj.lamb, time_scaling_factor=obj.time_scaling_factor,
                                 period_scaling_factor=obj.period_scaling_factor)

    @staticmethod
    def from_json_file(json_file_path: str) -> "SineArrivalConfig":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return SineArrivalConfig.from_dict(json.loads(json_str))



import datetime


class FailedLoginAttempt:
    """
    Class representing a failed login event on some server in the emulation
    """

    def __init__(self):
        """
        Initializes the object
        """
        self.timestamp = None

    @staticmethod
    def parse_from_str(login_attempt_str: str) -> "FailedLoginAttempt":
        """
        Parses a failed login event DTO from a string

        :param login_attempt_str: the string to parse
        :return: the parsed DTO
        """
        failed_login_attempt_dto = FailedLoginAttempt()
        failed_login_attempt_dto.timestamp = \
            datetime.datetime.strptime(login_attempt_str, '%Y %b %d %H:%M:%S').timestamp()
        return failed_login_attempt_dto

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return "timestamp:{}".format(self.timestamp)
